#####################################################  
## Direct Forecasting 
#####################################################
rm(list = ls(all = T)) 
library(GAMBoost)
library(mboost)
library(vars)
library(mda)
library(zoo)
source("../analysis/functions/helpers-forecasting.R")
source("../analysis/functions/helpers-general.R")
source("../analysis/functions/f-VAR.R")
print(load("../data/rdata/IP.RData"))
xchars <- print(load("../data/rdata/IP.indices.RData"))[-7]


#######  defaults #############################
setup <- list(ychar = "IPem",
              xchar = c("IPem", "ifo"),
              iniend = c(2003,12),
              maxLag = 12,
              hors = c(1,6,12))

#lsstp <- generate.setup(setup, xchars)
lsstp <- list(setup)

#######  parsets  #############################
#######  mboost   ############
parset.glmboost <- list(method = "glmboost",
                        mstop = 500,
                        dummy = FALSE)

parset.gamboost <- list(method = "gamboost",
                        df = 3.5,
                        knots = 20,
                        mstop = 500,
                        dummy = FALSE)

######   mda  ###############
parset.bruto <- list(method = "bruto",
                     cost = "log")

parset.mars  <- list(method = "mars",
                     penalty = "log")


######   VAR  ###############
parset.VARr <- list(method = "VAR",
                    maxLag = setup$maxLag,
                    restrict = TRUE,
                    hor = NULL,
                    iter = FALSE)

parset.VAR  <- list(method = "VAR",
                    maxLag = setup$maxLag,
                    restrict = FALSE,
                    hor = NULL,
                    iter = FALSE)

parset <- parset.glmboost

###### Forecasting ############################
dname <- make.names(format(Sys.time(), "%b %d %a %H %M %S"))
dirname <- paste("../res/fcstres/",dname,"/",sep="")
dir.create(dirname)
print(dirname)

### mboost
lsglm <- lapply(lsstp, function(z) dir.paradigm(z, parset.glmboost))
save(lsglm, file = paste(dirname, "lsglm.rda", sep=""))

lsgam <- lapply(lsstp, function(z) dir.paradigm(z, parset.gamboost))
save(lsgam, file = paste(dirname,"lsgam.rda",sep=""))

### mda
lsbruto <- lapply(lsstp, function(z) dir.paradigm(z, parset.bruto))
save(lsbruto, file = paste(dirname,"lsbruto.rda", sep=""))

lsmars <- lapply(lsstp, function(z) dir.paradigm(z, parset.mars))
save(lsmars, file = paste(dirname,"lsmars.rda", sep=""))

### VAR 
lsvarr <- lapply(lsstp, function(z) dir.paradigm(z, parset.VARr))
save(lsvarr, file = paste(dirname,"lsvarr.rda", sep=""))

lsvar <- lapply(lsstp, function(z) dir.paradigm(z, parset.VAR))
save(lsvar, file = paste(dirname,"lsvar.rda", sep=""))
