############################################################################
## simulation functions:   Huang and Yang, p. 470
############################################################################
## fe <- 0.9

AR1 <- function(theta1=0.5, theta2=0.4, len=100, post=12, fe) {  # theta1=0.5; theta2=0.4; len=100; post=12
 elen <- 400 + len + post     # more than 400 to converge
  y  <- rep(0,elen)
  y[1] <- fe*rnorm(1)
  y[2] <- y[1]*theta1 + fe*rnorm(1)
  for(i in 3:elen) y[i]  = theta1*y[i-1] + theta1*y[i-2] + fe*rnorm(1)
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L1","Series1.L2")
  res$truesetnum <- c(1,2)
  res$truepar <- c(0.5,0.4,rep(0,8))
  res
}


AR2 <- function(theta1=-0.5, theta2=0.4, len=100, post=12, fe){ # theta1=-0.5; theta2=0.4; len=100; post=12
 elen <- 400 + len + post     # more than 400 to converge
  y  <- rep(0,elen)
  y[1] <- fe*rnorm(1)
  y[2] <- y[1]*theta1 + fe*rnorm(1)
  for(i in 3:elen) y[i]  = theta1*y[i-1] + theta1*y[i-2] + fe*rnorm(1)
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L1","Series1.L2")
  res$truesetnum <- c(1,2)
  res$truepar <- c(-0.5,0.4,rep(0,8))
  res
}


AR3 <- function(theta1=-0.5, theta2=0.4, len=100, post=12, fe){ # theta1=-0.5; theta2=0.4; len=100; post=12
  elen <- 400 + len + post     # more than 400 to converge
  y  <- rep(0,elen)
  y[1:5] <- fe*rnorm(5)
  for(i in 7:10) y[i] <- y[i-6]*theta1 + fe*rnorm(1)
  for(i in 11:elen) y[i]  = theta1*y[i-6] + theta1*y[i-10] + fe*rnorm(1)
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L6","Series1.L10")
  res$truesetnum <- c(6,10)
  res$truepar <- c(0,0,0,0,0,-0.5,0,0,0,0.4)
  res
}


AR4 <- function(theta1=-0.8, theta2=-0.4, theta3=0.25, len=100, post=12, fe){
  elen <- 400+len+post
  y    <- rep(0,elen)
  y[1] <- fe*rnorm(1)
  y[2] <- theta1*y[1] + fe*rnorm(1)
  y[3] <- theta1*y[2] + theta2*y[1] + fe*rnorm(1)
  for(i in 4:elen) y[i] = theta1*y[i-1] + theta2*y[i-2] + theta3*y[i-3] + fe*rnorm(1)
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L1","Series1.L2","Series1.L3")
  res$truesetnum <- c(1,2,3)
  res$truepar <- c(theta1,theta2,theta3,rep(0,7))
  res
}


###################### NON LINEAR ######################
###==================================== 
### Huang and Yang, (2004)
###==================================== 
NLAR1a <- function(len=100,post=12, fe) {
   elen <- 400+len+post
   y <- rep(0,elen) 
   nlar1a.1 <- function(x)  -0.4*(3-x^2)/(1+x^2)
   cst <- function(x) rep(0,length(x))

   for(i in 2:elen) y[i] <- nlar1a.1(y[i-1]) + fe*rnorm(1)
   res = list(y = y[401:(400+len)])
   res$post = y[(401+len):elen]
   res$trueset <- c("Series1.L1")
   res$truesetnum <- c(1)
   res$truefct <- list(nlar1a.1,cst,cst,cst,cst,cst,cst,cst,cst,cst)
   res
  }


###==================================== 
### Huang and Yang, (2004)
###==================================== 
NLAR1b <- function(len=100,post=12, fe) {
   elen <- 400 + len + post
   y <- rep(0,elen) 
   nlar1b.2 <- function(x)  0.6*(3-(x-0.5)^3)/(1+(x-0.5)^4)
   cst <- function(x) rep(0,length(x))

   y[1:2] <- rnorm(2)
   for(i in 3:elen) y[i] <- nlar1b.2(y[i-2]) + fe*rnorm(1)
   res <- list(y = head(tail(y,len+post),len))
   res$post <- tail(y, post)
   res$trueset <- c("Series1.L2")
   res$truesetnum <- c(2)
   res$truefct <- list(cst,nlar1b.2,cst,cst,cst,cst,cst,cst,cst,cst)
   res
  }


###==================================== 
### Huang and Yang, (2004)
###==================================== 
NLAR2a <- function(len=100,post=12, fe) { 
   elen <- 400 + len + post
   y <- rep(0,elen) 
   nlar2a.1 <- function(x) -0.4*(3 - x^2)/(1+x^2)
   nlar2a.2 <- function(x)  0.6*(3-(x-0.5)^3)/(1+(x-0.5)^4)
   cst <- function(x) rep(0,length(x))

   y[1] <- rnorm(1)
   y[2] <- nlar2a.1(y[1]) + fe*rnorm(1)
   for(i in 3:elen) y[i] <- nlar2a.1(y[i-1]) + nlar2a.2(y[i-2]) + fe*rnorm(1)  
   res = list(y = y[401:(400+len)])
   res$post <- tail(y, post)
   res$trueset <- c("Series1.L1","Series1.L2")
   res$truesetnum <- c(1,2)
   res$truefct <- list(nlar2a.1,nlar2a.2,cst,cst,cst,cst,cst,cst,cst,cst)
   res
  }

###==================================== 
### Huang and Yang, (2004)
###==================================== 
NLAR2b <- function(len=100,post=12, fe) {
   elen <- 400+len+post
   y <- rep(0,len) 
   nlar2b.6 <- function(x)  (0.4 - 2*exp(-50*x^2))*x
   nlar2b.10 <- function(x) (0.5 - 0.5*exp(-50*x^2))*x
   cst <- function(x) rep(0,length(x))

   y[1:6] <- rnorm(6)
   for(i in 7:10) y[i] <- nlar2b.6(y[i-6]) + fe*rnorm(1)
   for(i in 11:elen) y[i]<- nlar2b.6(y[i-6]) + nlar2b.10(y[i-10]) + fe*rnorm(1)

   res = list(y = y[401:(400+len)])
   res$post = y[(401+len):elen]
   res$trueset <- c("Series1.L6","Series1.L10")
   res$truesetnum <- c(6,10)
   res$truefct <- list(cst,cst,cst,cst,cst,nlar2b.6,cst,cst,cst,nlar2b.10)
   res
}


###==================================== 
### Chen, R., Tsay, R., (1993), p.958,
###==================================== 
NLAR2c <- function(len=100,post=12, fe) {
  elen <- 400+len+post
  y <- rep(0,elen) 
  nlar2c.1 <- function(x) 0.8*log(1 + 3*x^2) 
  nlar2c.3 <- function(x) -0.6*log(1+3*x^2)
  cst <- function(x) rep(0,length(x))
  
  y[1:2] <- rnorm(2)
  y[3] <- nlar2c.1(y[2]) + rnorm(1)
  for(i in 4:elen) y[i] <- nlar2c.1(y[i-1]) + nlar2c.3(y[i-3]) + fe*rnorm(1) # orig. without fe
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L1","Series1.L3")
  res$truesetnum <- c(1,3)
  res$truefct <- list(nlar2c.1,cst,nlar2c.3,cst,cst,cst,cst,cst,cst,cst)
  res
}



###==================================== 
### Chen, R., Tsay, R., (1993), p.958,
###==================================== 
NLAR2d <- function(len=100,post=12, fe) {
  elen <- 400+len+post
  y <- rep(0,elen) 
  nlar2d.2 <- function(x) 1.5*sin((pi/2)*x)
  nlar2d.3 <- function(x) -sin((pi/2)*x)
  cst <- function(x) rep(0,length(x))
  
  y[1:2] <- rnorm(2)
  y[3] <- nlar2d.2(y[1]) + rnorm(1)
  for(i in 4:elen) y[i] <- nlar2d.2(y[i-2]) + nlar2d.3(y[i-3]) + fe*rnorm(1) # orig. without fe
  res = list(y = y[401:(400+len)])
  res$post = y[(401+len):elen]
  res$trueset <- c("Series1.L2","Series1.L3")
  res$truesetnum <- c(2,3)
  res$truefct <- list(cst,nlar2d.2,nlar2d.3,cst,cst,cst,cst,cst,cst,cst)
  res
}



###==================================== 
### Huang and Yang, (2004)
###==================================== 
NLAR2e <- function(len=100,post=12, fe) { 
   elen <- 400+len+post
   y <- rep(0,elen)
   nlar2e.6  <- function(x) (0.4 - 2*cos(40*x)*exp(-30*x^2))*x
   nlar2e.10 <- function(x) (0.55 - 0.55*sin(40*x)*exp(-10*x^2))*x
   cst <- function(x) rep(0,length(x))
   
   y[1:6] <- rnorm(6)
   for(i in 7:10) y[i] <- nlar2e.6(y[i-6]) + fe*rnorm(1)
   for(i in 11:elen) y[i]<- nlar2e.6(y[i-6]) + nlar2e.10(y[i-10]) + fe*rnorm(1)

   res = list(y = y[401:(400+len)])
   res$post = y[(401+len):elen]
   res$trueset <- c("Series1.L6","Series1.L10")
   res$truesetnum <- c(6,10)
   res$truefct <- list(cst,cst,cst,cst,cst,nlar2e.6,cst,cst,cst,nlar2e.10)
   res
  }


###==================================== 
### Tutz, Shafik (2007)
###==================================== 
NLAR4 <- function(len=100, post=12, fe) {  
   elen <- 400+len+post
   y <- rep(0,elen) 
   nlar4.4 <- function(x)  0.9*sin((pi/8)*x)
   nlar4.5 <- function(x) -0.75*sin((pi/8)*x)
   nlar4.6 <- function(x)  0.52*sin((pi/8)*x)
   nlar4.7 <- function(x)  0.38*sin((pi/8)*x) 
   cst <- function(x) rep(0,length(x))

   y[1:4] <- fe*rnorm(4)
   y[5] <- nlar4.4(y[1]) + fe*rnorm(1)
   y[6] <- nlar4.4(y[2]) + nlar4.5(y[1]) + fe*rnorm(1)
   y[7] <- nlar4.4(y[3]) + nlar4.5(y[2]) + nlar4.6(y[1]) + fe*rnorm(1)
   for(i in 8:elen)  y[i] = nlar4.4(y[i-4]) + nlar4.5(y[i-5]) + nlar4.6(y[i-6]) + nlar4.7(y[i-7]) + fe*rnorm(1)
   res = list(y = y[401:(400+len)])
   res$post = y[(401+len):elen]
   res$trueset <- c("Series1.L4","Series1.L5","Series1.L6","Series1.L7")
   res$truesetnum <- c(4,5,6,7)
   res$truefct <- list(cst,cst,cst,nlar4.4,nlar4.5,nlar4.6,nlar4.7,cst,cst,cst)
   res
}






##################################################
##VAR1 = function(len, post=3, intercept=F){
## elen = 400 + len + post
## require(MASS)
## if(intercept) nu = c(.02,.03) else nu = c(0,0)
## A1 = matrix(c(.5,.4,.1,.5),ncol=2)
## A2 = matrix(c(0,.25,0,0),ncol=2)
## Sigma = matrix(c(0.09,0,0,.04),ncol=2)
## Y = matrix(rep(c(0,0),elen),ncol=2)
## Y[1,] = mvrnorm(1,c(0,0),Sigma)
## Y[2,] = nu + A1%*%Y[1,] + mvrnorm(1,c(0,0),Sigma)
## Y[3,] = A1%*%Y[2,] + A2%*%Y[1,] + mvrnorm(1,c(0,0),Sigma)
## for(i in 4:elen) 
##  Y[i,] = A1%*%Y[i-1,] + A2%*%Y[i-2,] + mvrnorm(1,c(0,0),Sigma)
## res = list(y=Y[401:(400+len),]) 
## res$post = Y[(401+len):elen,]
## res$param = cbind(A1,A2,matrix(0,nrow=2,ncol=16))
## res$ord = 2
## res
##}
##################################################
