#################################################################
### monte-carlo simulations:
#################################################################
rm(list = ls())
source("c:/acer-book/book_all/nikolay_robinzonov_boost-Rcode/boost_nlar/analysis/functions/helpers-general.R") # simulate time series
source("c:/acer-book/book_all/nikolay_robinzonov_boost-Rcode/boost_nlar/analysis/functions/helpers-mc.R") # simulate time series
source("c:/acer-book/book_all/nikolay_robinzonov_boost-Rcode/boost_nlar/analysis/functions/stepwise.R")        # stepwise algorithm 

library(mboost)
library(GAMBoost)
library(mda)


###==================== preparation ====================
# myrepo <- "../datawork/repository"           # point to repos
myrepo <- "c:/acer-book/book_all/nikolay_robinzonov_boost-Rcode/boost_nlar/datawork/repository"

xd <- dir(myrepo)
strd.ts <- xd[grep("\\.RData$",xd)]     # RData files only
dname <- make.names(format(Sys.time(), "%b %d %a %H %M %S"))
dirname <- paste("c:/acer-book/book_all/nikolay_robinzonov_boost-Rcode/boost_nlar/datawork/simres/",dname,"/",sep="")
print(dirname)
dir.create(dirname)

###==================== setup (simulation specific) ====
setup <- list(repo = myrepo,          # repository
              stored.ts = strd.ts,      # supply with sim data
              dsimres = dirname,        # where to store in
              d = 10,                   # maximal lag
              n = 100)                  # number of TSs
rm(xd, myrepo, strd.ts, dname)

###==================== parameters (model specific) ====
parset.gamboost <- list(method = "gamboost",
                        df = 3.5,
                        knots = 20,
                        mstop = 500)

parset.mars  <- list(method = "mars",
                     penalty = "log")

parset.bruto <- list(method = "bruto",
                     cost = "log")

parset.HaY <- list(method = "HaY",
                   Smax = 10,
                   basis = "cr")


parset <- parset.gamboost


###===================== computation ===================
res <- mc.paradigm(setup, parset.mars)
fileName <- paste(dirname,paste(unlist(parset.mars),collapse="_"),".RData",sep="")
save(res, file = fileName)

res <- mc.paradigm(setup, parset.bruto)
fileName <- paste(dirname,paste(unlist(parset.bruto),collapse="_"),".RData",sep="")
save(res, file = fileName)


res <- mc.paradigm(setup, parset.HaY)
fileName <- paste(dirname,paste(unlist(parset.HaY),collapse="_"),".RData",sep="")
save(res, file = fileName)


res <- mc.paradigm(setup, parset.gamboost)
fileName <- paste(dirname,paste(unlist(parset.gamboost),collapse="_"),".RData",sep="")
save(res, file = fileName)